@@ -1,7 +1,7 @@
 package Perinci::Examples::Stream;
 
-our $DATE = '2016-01-22'; # DATE
-our $VERSION = '0.68'; # VERSION
+our $DATE = '2016-02-04'; # DATE
+our $VERSION = '0.69'; # VERSION
 
 use 5.010;
 use strict;
@@ -19,26 +19,30 @@ This package contains functions that demonstrate streaming input/output.
 _
 };
 
+my %arg_num = (
+    num => {
+        summary => 'Limit number of entries to produce',
+        description => <<'_',
+
+The default is to produce an infinite number.
+
+_
+        schema => ['int*', min=>0],
+        cmdline_aliases => {n=>{}},
+        pos => 0,
+    },
+);
+
 $SPEC{produce_ints} = {
     v => 1.1,
     summary => 'This function produces a stream of integers, starting from 1',
     tags => ['category:streaming-result'],
     args => {
-        num => {
-            summary => 'Limit number of numbers to produce',
-            description => <<'_',
-
-The default is to produce an infinite number.
-
-_
-            schema => 'int*',
-            cmdline_aliases => {n=>{}},
-            pos => 0,
-        },
+        %arg_num,
     },
     result => {
         stream => 1,
-        schema => 'int*',
+        schema => ['array*', of=>'int*'],
     },
 };
 sub produce_ints {
@@ -57,8 +61,8 @@ $SPEC{count_ints} = {
     tags => ['category:streaming-input'],
     args => {
         input => {
-            summary => 'Number',
-            schema => 'int*',
+            summary => 'Numbers',
+            schema => ['array*', of=>'int*'],
             stream => 1,
             cmdline_src => 'stdin_or_files',
         },
@@ -69,7 +73,7 @@ sub count_ints {
     my $input = $args{input};
     my $n = 0;
     $n++ while defined $input->();
-    [200, "OK", "You input $n int(s)"];
+    [200, "OK", $n];
 }
 
 $SPEC{count_lines} = {
@@ -78,8 +82,8 @@ $SPEC{count_lines} = {
     tags => ['category:streaming-input'],
     args => {
         input => {
-            summary => 'Input',
-            schema => 'str*',
+            summary => 'Lines',
+            schema => ['array*', of=>'str*'],
             stream => 1,
             cmdline_src => 'stdin_or_files',
         },
@@ -98,16 +102,11 @@ $SPEC{produce_words} = {
     summary => 'This function produces a stream of random words',
     tags => ['category:streaming-result'],
     args => {
-        num => {
-            summary => 'Limit number of words to produce',
-            schema => 'int*',
-            cmdline_aliases => {n=>{}},
-            pos => 0,
-        },
+        %arg_num,
     },
     result => {
         stream => 1,
-        schema => ['str*', match=>'\A\w+\z'],
+        schema => ['array*', of=>['str*', match=>'\A\w+\z']],
     },
 };
 sub produce_words {
@@ -126,15 +125,11 @@ $SPEC{produce_words_err} = {
     summary => 'Like `produce_words()`, but 1 in every 10 words will be a non-word (which fails the result schema)',
     tags => ['categoryr:streaming-result'],
     args => {
-        num => {
-            summary => 'Limit number of words to produce',
-            schema => 'int*',
-            cmdline_aliases => {n=>{}},
-        },
+        %arg_num,
     },
     result => {
         stream => 1,
-        schema => ['str*', match=>'\A\w+\z'],
+        schema => ['array*', of => ['str*', match=>'\A\w+\z']],
     },
 };
 sub produce_words_err {
@@ -161,8 +156,8 @@ Input validation will check that each record from the stream is a word.
 
 _
     args => {
-        words => {
-            schema => ['str*', match=>'\A\w+\z'],
+        input => {
+            schema => ['array*', of=>['str*', match=>'\A\w+\z']],
             stream => 1,
             cmdline_src => 'stdin_or_files',
         },
@@ -171,9 +166,9 @@ _
 sub count_words {
     my %args = @_;
 
-    my $words = $args{words};
+    my $input = $args{input};
     my $num = 0;
-    while (defined($words->())) {
+    while (defined($input->())) {
         $num++;
     }
     [200, "OK", $num];
@@ -183,15 +178,11 @@ $SPEC{produce_hashes} = {
     v => 1.1,
     summary => 'This function produces a stream of hashes',
     args => {
-        num => {
-            schema => 'int*',
-            cmdline_aliases => {n=>{}},
-            pos => 0,
-        },
+        %arg_num,
     },
     result => {
         stream => 1,
-        schema => 'hash*',
+        schema => ['array*', of=>'hash*'],
     },
 };
 sub produce_hashes {
@@ -213,13 +204,13 @@ $SPEC{square_nums} = {
         input => {
             req => 1,
             stream => 1,
-            schema => 'float*',
+            schema => ['array*', of=>'float*'],
             cmdline_src => 'stdin_or_files',
         },
     },
     result => {
         stream => 1,
-        schema => 'float*',
+        schema => ['array*', of=>'float*'],
     },
 };
 sub square_nums {
@@ -242,13 +233,13 @@ $SPEC{square_nums_from_file} = {
             req => 1,
             pos => 0,
             stream => 1,
-            schema => 'float*',
+            schema => ['array*', of=>'float*'],
             cmdline_src => 'file',
         },
     },
     result => {
         stream => 1,
-        schema => 'float*',
+        schema => ['array*', of=>'float*'],
     },
 };
 sub square_nums_from_file {
@@ -264,13 +255,13 @@ $SPEC{square_nums_from_stdin} = {
             req => 1,
             pos => 0,
             stream => 1,
-            schema => 'float*',
+            schema => ['array*', of=>'float*'],
             cmdline_src => 'stdin',
         },
     },
     result => {
         stream => 1,
-        schema => 'float*',
+        schema => ['array*', of=>'float*'],
     },
 };
 sub square_nums_from_stdin {
@@ -292,7 +283,7 @@ $SPEC{square_nums_from_stdin_or_file} = {
     },
     result => {
         stream => 1,
-        schema => 'float*',
+        schema => ['array*', of=>'float*'],
     },
 };
 sub square_nums_from_stdin_or_file {
@@ -307,7 +298,7 @@ $SPEC{wc} = {
         input => {
             req => 1,
             stream => 1,
-            schema => 'str*',
+            schema => ['array*', of=>'str*'],
             cmdline_src => 'stdin_or_files',
             'cmdline.chomp' => 0,
         },
@@ -343,7 +334,7 @@ _
         input => {
             req => 1,
             stream => 1,
-            schema => 'hash*',
+            schema => ['array*', of=>'hash*'],
             cmdline_src => 'stdin_or_files',
         },
     },
@@ -377,7 +368,7 @@ Perinci::Examples::Stream - Examples for streaming input/output
 
 =head1 VERSION
 
-This document describes version 0.68 of Perinci::Examples::Stream (from Perl distribution Perinci-Examples), released on 2016-01-22.
+This document describes version 0.69 of Perinci::Examples::Stream (from Perl distribution Perinci-Examples), released on 2016-02-04.
 
 =head1 DESCRIPTION
 
@@ -397,9 +388,9 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
-=item * B<input> => I<int>
+=item * B<input> => I<array[int]>
 
-Number.
+Numbers.
 
 =back
 
@@ -425,9 +416,9 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
-=item * B<input> => I<str>
+=item * B<input> => I<array[str]>
 
-Input.
+Lines.
 
 =back
 
@@ -455,7 +446,7 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
-=item * B<words> => I<str>
+=item * B<input> => I<array[str]>
 
 =back
 
@@ -483,33 +474,7 @@ Arguments ('*' denotes required arguments):
 
 =item * B<num> => I<int>
 
-=back
-
-Returns an enveloped result (an array).
-
-First element (status) is an integer containing HTTP status code
-(200 means OK, 4xx caller error, 5xx function error). Second element
-(msg) is a string containing error message, or 'OK' if status is
-200. Third element (result) is optional, the actual result. Fourth
-element (meta) is called result metadata and is optional, a hash
-that contains extra information.
-
-Return value:  (hash)
-
-
-=head2 produce_ints(%args) -> [status, msg, result, meta]
-
-This function produces a stream of integers, starting from 1.
-
-This function is not exportable.
-
-Arguments ('*' denotes required arguments):
-
-=over 4
-
-=item * B<num> => I<int>
-
-Limit number of numbers to produce.
+Limit number of entries to produce.
 
 The default is to produce an infinite number.
 
@@ -524,7 +489,37 @@ First element (status) is an integer containing HTTP status code
 element (meta) is called result metadata and is optional, a hash
 that contains extra information.
 
-Return value:  (int)
+Return value:  (array[hash])
+
+
+=head2 produce_ints(%args) -> [status, msg, result, meta]
+
+This function produces a stream of integers, starting from 1.
+
+This function is not exportable.
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<num> => I<int>
+
+Limit number of entries to produce.
+
+The default is to produce an infinite number.
+
+=back
+
+Returns an enveloped result (an array).
+
+First element (status) is an integer containing HTTP status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+(msg) is a string containing error message, or 'OK' if status is
+200. Third element (result) is optional, the actual result. Fourth
+element (meta) is called result metadata and is optional, a hash
+that contains extra information.
+
+Return value:  (array[int])
 
 
 =head2 produce_words(%args) -> [status, msg, result, meta]
@@ -539,7 +534,9 @@ Arguments ('*' denotes required arguments):
 
 =item * B<num> => I<int>
 
-Limit number of words to produce.
+Limit number of entries to produce.
+
+The default is to produce an infinite number.
 
 =back
 
@@ -552,7 +549,7 @@ First element (status) is an integer containing HTTP status code
 element (meta) is called result metadata and is optional, a hash
 that contains extra information.
 
-Return value:  (str)
+Return value:  (array[str])
 
 
 =head2 produce_words_err(%args) -> [status, msg, result, meta]
@@ -567,7 +564,9 @@ Arguments ('*' denotes required arguments):
 
 =item * B<num> => I<int>
 
-Limit number of words to produce.
+Limit number of entries to produce.
+
+The default is to produce an infinite number.
 
 =back
 
@@ -580,7 +579,7 @@ First element (status) is an integer containing HTTP status code
 element (meta) is called result metadata and is optional, a hash
 that contains extra information.
 
-Return value:  (str)
+Return value:  (array[str])
 
 
 =head2 square_nums(%args) -> [status, msg, result, meta]
@@ -593,7 +592,7 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
-=item * B<input>* => I<float>
+=item * B<input>* => I<array[float]>
 
 =back
 
@@ -606,7 +605,7 @@ First element (status) is an integer containing HTTP status code
 element (meta) is called result metadata and is optional, a hash
 that contains extra information.
 
-Return value:  (float)
+Return value:  (array[float])
 
 
 =head2 square_nums_from_file(%args) -> [status, msg, result, meta]
@@ -619,7 +618,7 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
-=item * B<input>* => I<float>
+=item * B<input>* => I<array[float]>
 
 =back
 
@@ -632,7 +631,7 @@ First element (status) is an integer containing HTTP status code
 element (meta) is called result metadata and is optional, a hash
 that contains extra information.
 
-Return value:  (float)
+Return value:  (array[float])
 
 
 =head2 square_nums_from_stdin(%args) -> [status, msg, result, meta]
@@ -645,7 +644,7 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
-=item * B<input>* => I<float>
+=item * B<input>* => I<array[float]>
 
 =back
 
@@ -658,7 +657,7 @@ First element (status) is an integer containing HTTP status code
 element (meta) is called result metadata and is optional, a hash
 that contains extra information.
 
-Return value:  (float)
+Return value:  (array[float])
 
 
 =head2 square_nums_from_stdin_or_file(%args) -> [status, msg, result, meta]
@@ -684,7 +683,7 @@ First element (status) is an integer containing HTTP status code
 element (meta) is called result metadata and is optional, a hash
 that contains extra information.
 
-Return value:  (float)
+Return value:  (array[float])
 
 
 =head2 wc(%args) -> [status, msg, result, meta]
@@ -697,7 +696,7 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
-=item * B<input>* => I<str>
+=item * B<input>* => I<array[str]>
 
 =back
 
@@ -726,7 +725,7 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
-=item * B<input>* => I<hash>
+=item * B<input>* => I<array[hash]>
 
 =back
 
